import { ScalewayError } from '../scw-error.js';

/**
 * TransientState error happens when trying to perform an action on a resource in a transient state.
 *
 * @public
 */
class TransientStateError extends ScalewayError {
  constructor(status, body, resource, resourceId, currentState) {
    super(status, body, `resource ${resource} with ID ${resourceId} is in a transient state: ${currentState}`);
    this.status = status;
    this.body = body;
    this.resource = resource;
    this.resourceId = resourceId;
    this.currentState = currentState;
    this.name = 'TransientStateError';
  }
  static fromJSON(status, obj) {
    if (typeof obj.resource !== 'string' || typeof obj.resource_id !== 'string' || typeof obj.current_state !== 'string') {
      return null;
    }
    return new TransientStateError(status, obj, obj.resource, obj.resource_id, obj.current_state);
  }
}

export { TransientStateError };
